@@ -24,8 +24,10 @@ import org.apache.flink.api.common.state.ListStateDescriptor;
 import org.apache.flink.api.common.state.MapStateDescriptor;
 import org.apache.flink.api.common.typeinfo.BasicTypeInfo;
 import org.apache.flink.ml.param.Param;
+import org.apache.flink.ml.servable.builder.ExampleServables.SumModelServable;
 import org.apache.flink.ml.util.ParamUtils;
 import org.apache.flink.ml.util.ReadWriteUtils;
+import org.apache.flink.ml.util.TestUtils;
 import org.apache.flink.runtime.state.StateInitializationContext;
 import org.apache.flink.streaming.api.datastream.DataStream;
 import org.apache.flink.streaming.api.operators.AbstractStreamOperator;
@@ -109,6 +111,10 @@ public class ExampleStages {
             SumModel model = ReadWriteUtils.loadStageParam(path);
             return model.setModelData(modelDataTable);
         }
+
+        public static SumModelServable loadServable(String path) throws IOException {
+            return SumModelServable.load(path);
+        }
     }
 
     // Adds delta from the 2nd input to every element in the 1st input and returns the added values.
