@@ -115,10 +115,10 @@ public class LogisticRegression
                 optimizer.optimize(initModelData, trainData, BinaryLogisticLoss.INSTANCE);
 
         DataStream<LogisticRegressionModelData> modelData =
-                rawModelData.map(vector -> new LogisticRegressionModelData(vector, 0));
+                rawModelData.map(vector -> new LogisticRegressionModelData(vector, 0L));
         LogisticRegressionModel model =
                 new LogisticRegressionModel().setModelData(tEnv.fromDataStream(modelData));
-        ReadWriteUtils.updateExistingParams(model, paramMap);
+        ParamUtils.updateExistingParams(model, paramMap);
         return model;
     }
 
