@@ -30,6 +30,7 @@ import org.apache.flink.metrics.Gauge;
 import org.apache.flink.ml.classification.logisticregression.LogisticRegression;
 import org.apache.flink.ml.classification.logisticregression.LogisticRegressionModel;
 import org.apache.flink.ml.classification.logisticregression.LogisticRegressionModelData;
+import org.apache.flink.ml.classification.logisticregression.LogisticRegressionModelDataUtil;
 import org.apache.flink.ml.classification.logisticregression.OnlineLogisticRegression;
 import org.apache.flink.ml.classification.logisticregression.OnlineLogisticRegressionModel;
 import org.apache.flink.ml.linalg.DenseVector;
@@ -38,6 +39,7 @@ import org.apache.flink.ml.linalg.Vectors;
 import org.apache.flink.ml.util.InMemorySinkFunction;
 import org.apache.flink.ml.util.InMemorySourceFunction;
 import org.apache.flink.ml.util.TestUtils;
+import org.apache.flink.runtime.client.JobStatusMessage;
 import org.apache.flink.runtime.jobgraph.JobGraph;
 import org.apache.flink.runtime.minicluster.MiniCluster;
 import org.apache.flink.runtime.minicluster.MiniClusterConfiguration;
@@ -52,8 +54,10 @@ import org.apache.flink.util.TestLogger;
 
 import org.apache.commons.collections.IteratorUtils;
 import org.junit.After;
+import org.junit.AfterClass;
 import org.junit.Assert;
 import org.junit.Before;
+import org.junit.BeforeClass;
 import org.junit.Rule;
 import org.junit.Test;
 import org.junit.rules.TemporaryFolder;
@@ -151,12 +155,10 @@ public class OnlineLogisticRegressionTest extends TestLogger {
     private InMemorySinkFunction<Row> outputSink;
     private InMemorySinkFunction<LogisticRegressionModelData> modelDataSink;
 
-    // TODO: creates static mini cluster once for whole test class after dependency upgrades to
-    // Flink 1.15.
-    private InMemoryReporter reporter;
-    private MiniCluster miniCluster;
-    private StreamExecutionEnvironment env;
-    private StreamTableEnvironment tEnv;
+    private static InMemoryReporter reporter;
+    private static MiniCluster miniCluster;
+    private static StreamExecutionEnvironment env;
+    private static StreamTableEnvironment tEnv;
 
     private Table offlineTrainDenseTable;
     private Table onlineTrainDenseTable;
@@ -166,17 +168,8 @@ public class OnlineLogisticRegressionTest extends TestLogger {
     private Table initDenseModel;
     private Table initSparseModel;
 
-    @Before
-    public void before() throws Exception {
-        currentModelDataVersion = 0;
-
-        trainDenseSource = new InMemorySourceFunction<>();
-        predictDenseSource = new InMemorySourceFunction<>();
-        trainSparseSource = new InMemorySourceFunction<>();
-        predictSparseSource = new InMemorySourceFunction<>();
-        outputSink = new InMemorySinkFunction<>();
-        modelDataSink = new InMemorySinkFunction<>();
-
+    @BeforeClass
+    public static void beforeClass() throws Exception {
         Configuration config = new Configuration();
         config.set(RestOptions.BIND_PORT, "18081-19091");
         config.set(ExecutionCheckpointingOptions.ENABLE_CHECKPOINTS_AFTER_TASKS_FINISH, true);
@@ -193,10 +186,23 @@ public class OnlineLogisticRegressionTest extends TestLogger {
         miniCluster.start();
 
         env = StreamExecutionEnvironment.getExecutionEnvironment(config);
+        env.getConfig().enableObjectReuse();
         env.setParallelism(defaultParallelism);
         env.enableCheckpointing(100);
         env.setRestartStrategy(RestartStrategies.noRestart());
         tEnv = StreamTableEnvironment.create(env);
+    }
+
+    @Before
+    public void before() throws Exception {
+        currentModelDataVersion = 0;
+
+        trainDenseSource = new InMemorySourceFunction<>();
+        predictDenseSource = new InMemorySourceFunction<>();
+        trainSparseSource = new InMemorySourceFunction<>();
+        predictSparseSource = new InMemorySourceFunction<>();
+        outputSink = new InMemorySinkFunction<>();
+        modelDataSink = new InMemorySinkFunction<>();
 
         offlineTrainDenseTable =
                 tEnv.fromDataStream(env.fromElements(TRAIN_DENSE_ROWS_1)).as("features", "label");
@@ -265,6 +271,13 @@ public class OnlineLogisticRegressionTest extends TestLogger {
 
     @After
     public void after() throws Exception {
+        for (JobStatusMessage message : miniCluster.listJobs().get()) {
+            miniCluster.cancelJob(message.getJobId());
+        }
+    }
+
+    @AfterClass
+    public static void afterClass() throws Exception {
         miniCluster.close();
     }
 
@@ -281,7 +294,7 @@ public class OnlineLogisticRegressionTest extends TestLogger {
         tEnv.toDataStream(outputTable).addSink(outputSink);
 
         Table modelDataTable = onlineModel.getModelData()[0];
-        LogisticRegressionModelData.getModelDataStream(modelDataTable).addSink(modelDataSink);
+        LogisticRegressionModelDataUtil.getModelDataStream(modelDataTable).addSink(modelDataSink);
     }
 
     /** Blocks the thread until Model has set up init model data. */
@@ -500,7 +513,8 @@ public class OnlineLogisticRegressionTest extends TestLogger {
 
     @Test
     public void testGenerateRandomModelData() throws Exception {
-        Table modelDataTable = LogisticRegressionModelData.generateRandomModelData(tEnv, 2, 2022);
+        Table modelDataTable =
+                LogisticRegressionModelDataUtil.generateRandomModelData(tEnv, 2, 2022);
         DataStream<Row> modelData = tEnv.toDataStream(modelDataTable);
         Row modelRow = (Row) IteratorUtils.toList(modelData.executeAndCollect()).get(0);
         Assert.assertEquals(2, ((DenseVector) modelRow.getField(0)).size());
@@ -549,8 +563,6 @@ public class OnlineLogisticRegressionTest extends TestLogger {
     @Test
     public void testBatchSizeLessThanParallelism() {
         try {
-            env.setParallelism(defaultParallelism);
-            trainDenseSource.addAll(TRAIN_DENSE_ROWS_1);
             new OnlineLogisticRegression()
                     .setInitialModelData(initDenseModel)
                     .setReg(0.2)
