@@ -25,6 +25,7 @@ import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.table.api.Table;
 import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
 import org.apache.flink.test.util.AbstractTestBase;
+import org.apache.flink.test.util.TestBaseUtils;
 import org.apache.flink.types.Row;
 
 import org.apache.commons.collections.IteratorUtils;
@@ -97,7 +98,7 @@ public class BucketizerTest extends AbstractTestBase {
             result.add(outRow);
         }
 
-        compareResultCollections(
+        TestBaseUtils.compareResultCollections(
                 expectedResult, result, Comparator.comparingInt(r -> ((Integer) r.getField(0))));
     }
 
