@@ -25,6 +25,7 @@ import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;
 import org.apache.flink.table.api.Table;
 import org.apache.flink.table.api.bridge.java.StreamTableEnvironment;
 import org.apache.flink.test.util.AbstractTestBase;
+import org.apache.flink.test.util.TestBaseUtils;
 import org.apache.flink.types.Row;
 
 import org.apache.commons.collections.IteratorUtils;
@@ -421,7 +422,7 @@ public class StringIndexerTest extends AbstractTestBase {
     }
 
     static void verifyPredictionResult(List<Row> expected, List<Row> result) {
-        compareResultCollections(
+        TestBaseUtils.compareResultCollections(
                 expected,
                 result,
                 (row1, row2) -> {
